library(moments)
library(gsg)
library(mgcv)
library(ggplot2)

### a) Environmental skewness 
##############################################

### Breeding values distribution
l_skewe = 0.1
g_skewe <- rnorm(1000, mean = 0, sd = sqrt(l_skewe))
gbar_skewe = 0
DOpt_skewe = 0
### Environmental values distribution
i_skewe = 2
ve_skewe = 1
k_skewe = ((2/i_skewe)^2) 
theta_skewe = (sqrt(ve_skewe)/sqrt(k_skewe))
3e_skewe = (i_skewe*(sqrt(ve_skewe)^3))

e_skewe <- rgamma(1000, shape = k_skewe, scale = theta_skewe)
e_skewe <- e_skewe - mean(e_skewe)

### Phenotypic distribution  
z_skewe <- g_skewe + e_skewe

### b) Genetic skewness
##############################################

### Breeding value distribution
i_skewg = 2
vg_skewg = 1  
k_skewg = ((2/i_skewg)^2) 
theta_skewg = (sqrt(vg_skewg)/sqrt(k_skewg)) 
3g_skewg = (i_skewg*(sqrt(vg_skewg)^3)) 

g_skewg <- rgamma(1000, shape = k_skewg, scale = theta_skewg)

###  Position of the mean breeding values (= phenotypic mean) mean from the adaptive optimum (DOpt)
DOpt_skewg = (-((vg_skewg)^2)/(sqrt(k_skewg)))
opt_skewg = 0 
gbar_skewg = DOpt_skewg + opt_skewg
g_skewg <- g_skewg - mean(g_skewg) + gbar_skewg 

### Phenotypic distribution 
l_skewg = 0.1
e_skewg <- rnorm(1000, mean = 0, sd = sqrt(l_skewg))
z_skewg <- g_skewg + e_skewg 

### Fitness function 
##############################################
wmax = 1 # maximal fitness 
om = 5 # peak width of the fitness function 
opt = 0 # adaptive optimum

# with Environmental skewness 
funcfit_skewe <- function(z_skewe) {   
  (wmax*(1-(((z_skewe-opt)^2)/(2*(om^2)))))
}
# with Genetic skewness
funcfit_skewg <- function(z_skewg) {   
  (wmax*(1-(((z_skewg-opt)^2)/(2*(om^2)))))
}

### Directional selection gradients
##############################################
# Environmental skewness 
# Fitness of individuals
fit_skewe <- (wmax*(1-(((z_skewe-opt)^2)/(2*(om^2)))))
fitrel_skewe <- fit_skewe/mean(fit_skewe)
# OLS
regBetaEst_skewe =  lm(fitrel_skewe~z_skewe)

# Genetic skewness 
# Fitness of individuals
fit_skewg <- (wmax*(1-(((z_skewg-opt)^2)/(2*(om^2)))))
fitrel_skewg <- fit_skewg/mean(fit_skewg)
# OLS
regBetaEst_skewg =  lm(fitrel_skewg~z_skewg)

# Graphs
##############################################
par(mfrow=c(2,1))

histdata <- hist(z_skewe,breaks=100,freq=F,axes=T,add=F,xlim=c(-4,8),ylim=c(-0.07,1), 
                 xlab="Phenotype",ylab="Fitness",main="a) Skewness in environmental effects", 
                 col="lightgrey")
curve(funcfit_skewe, add=T, lwd = 1, xlim=c(-4,6))
abline(regBetaEst_skewe, col="black", lty = 1, lwd = 1.5)
segments(0, 1, x1 = 0, y1 = 0, col="black",lty = 3, lwd = 2)
mean_z_skew_e <- mean(z_skewe)
segments(mean_z_skew_e, 1, x1 = 0, y1 = 0, lty = 5, lwd = 3, col="black")
text(0, -0.05, expression(bar(z) == theta))
mean_z_skew_e <- mean(z_skewe)

histdata <- hist(z_skewg,breaks=100,freq=F,axes=T,add=F,xlim=c(-4,8),ylim=c(-0.07,1), 
                 xlab="Breeding values",ylab="Fitness",main="b) Skewness in breeding values", 
                 col="lightgrey")
curve(funcfit_skewg, add=T, lwd = 1, xlim=c(-4,6))
abline(regBetaEst_skewg, col="black", lty = 1, lwd = 1.5)
segments(0, 1, x1 = 0, y1 = 0, col="black",lty = 3, lwd = 2)
mean_g_skew_g <- mean(g_skewg)
segments(mean_g_skew_g, 0.982, x1 = mean_g_skew_g, y1 = 0, lty = 5, lwd = 3, col="black")
text(0, -0.05, expression(theta))
text(mean_g_skew_g, -0.06, expression(bar(g) == bar(z)))
