library(moments)
library(gsg)
library(mgcv)
library(ggplot2)
library(moments)
library(ggplot2)
library(lattice)
library(Rmisc)
library(reshape2)
library(dgof)
library(gridExtra)
library(grid)

NbSimul <- c(1:100) # number of replicats 
valAsym <- c(0.0001:10) # skewness of genetic distribution  
valSd <- c(0.00001, 1, 2, 5) # environmental variance 

data <- NULL

for (i in valAsym){
  for (l in valSd){
    for (p in 1:length(NbSimul)){
      
      vg = 1 # genetic variance 
      k = ((2/i)^2) # Gamma distribution parameters (shape)
      theta = (sqrt(vg)/sqrt(k)) # Gamma distribution parameters (scale)
      3g = (i*(sqrt(vg)^3)) # Third central moment of the breeding values distribution
      g <- rgamma(1000, shape = k, scale = theta)
      g <- g - mean(g)
      
      ### Phenotypic distribution  
      e <- rnorm(1000, mean = 0, sd = sqrt(l))
      z <- g + e 
      
      ### Realized parameters
      vg_real = var(g) # realized variance of genetic values
      ve_real = var(e) # realized variance of environmental distribution
      vz_real = var(z) # realized variance of phenotypic values 
      skew_g_real = skewness(g) # realized skewness of breeding values
      skew_e_real = skewness(e) # realized third central moment of the environmental values distribution
      skew_z_real = skewness(z) # realized phenotypic third central moment
      3g_real = (skew_g_real*(sqrt(vg_real)^3)) # realized third central moment of breeding values
      3e_real = (skew_e_real*(sqrt(ve_real)^3)) # realized third central moment of environmental values
      3z_real = (skew_z_real*(sqrt(vz_real)^3)) # realized third central moment of phenotypic values 
      vec <- c(i, l, p, skew_g_real, skew_e_real, skew_z_real)
      data <- rbind(data, vec)
    }   
  }
}

data <- as.data.frame(data)

names(data)[1] <- "skew_g"; names(data)[2] <- "ve"; names(data)[3] <- "repl"; names(data)[4] <- "skew_g_real"; names(data)[5] <- "skew_e_real"; names(data)[6] <- "skew_z_real"

hh <- as.numeric(as.numeric(data$ve))
hh[hh%in% c(1)]<-0.5
hh[hh%in% c(0.00001)]<-1
hh[hh%in% c(2)]<-0.33
hh[hh%in% c(5)]<-0.16
data <- cbind(data, hh)

dt<-aggregate(data, by = list(data$hh, data$skew_g), FUN=mean)
dt2<-aggregate(data, by = list(data$hh, data$skew_g), FUN=sd)
dt<-dt[,!names(dt)%in%c("hh", "skew_g")]
dt2<-dt2[,!names(dt2)%in%c("hh", "skew_g")]
names(dt)[1:2]<-c("hh", "skew_g")
names(dt2)[1:2]<-c("hh", "skew_g")
dt<-merge(dt,dt2, by = c("hh", "skew_g"))

corr <- function(x){ 
  x
}

ggplot(dt, aes(x=skew_g, y=(skew_z_real.x), color = factor(hh), labeller = "hh", solid=FALSE), geom = c("jitter")) +
  geom_errorbar(aes(ymin=skew_z_real.x-skew_z_real.y, ymax=skew_z_real.x+skew_z_real.y), width=0.13, size = 1) +
  geom_point(size=5, shape = 20) +
  theme_classic() + scale_shape(solid = FALSE) +
  stat_function(fun = corr, colour = "#000000", size = 0.6, linetype = "dashed") +
  scale_colour_grey(start = 0.8, end = 0.3) +
  ylim(-0.2, 10.5) +
  xlim(-0.2, 10) +
  labs(list(color = "h", title = "",
            x = "Skewness of breeding values", y = "Skewness of phenotype")) +
  theme(plot.title = element_text(size = 15), legend.position='none', axis.title.y = element_text(size = 12, angle = 90),
        axis.title.x = element_text(size = 12, angle = 0), axis.text.x  = element_text(vjust=0.5, size=10), 
        axis.text.y  = element_text(vjust=0.5, size=10))
